"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const typescript = require("typescript");
class AngularResourceAnalyzer {
    constructor(filePath, fileContent) {
        this.externalResources = [];
        this.filePath = filePath;
        this.fileName = path_1.posix.basename(filePath);
        this.fileContent = fileContent;
        try {
            this.sourceFile = typescript.createSourceFile(this.fileName, this.fileContent, typescript.ScriptTarget.Latest, true);
        }
        catch (error) {
            throw new Error([
                `An error occured while trying to parse the file "${this.fileName}" as a TypeScript source file.`,
                error.message
            ].join('\n'));
        }
    }
    findExternalResources() {
        this.findComponentDecorators(this.sourceFile);
        return this.externalResources;
    }
    inlineExternalResources(externalResources) {
        return externalResources
            .reverse()
            .reduce((fileContent, externalResource) => {
            fileContent = externalResource.urls
                .reverse()
                .reduce((fileContent, url) => {
                return this.replaceNodeContent(fileContent, `\`${url.content}\``, url.node);
            }, fileContent);
            fileContent = this.replaceNodeContent(fileContent, externalResource.newKey, externalResource.node);
            return fileContent;
        }, this.fileContent);
    }
    findComponentDecorators(node) {
        if (this.isComponentDecorator(node)) {
            this.findExternalTemplatesAndStyles(node);
        }
        typescript.forEachChild(node, this.findComponentDecorators.bind(this));
    }
    findExternalTemplatesAndStyles(node) {
        if (this.isTemplateUrlDeclaration(node)) {
            const templateUrlKeyNode = node.name;
            const templateUrlValueNode = node.initializer;
            const templateUrlValue = templateUrlValueNode.text;
            if (templateUrlValue === '' || templateUrlValue === 'undefined' || templateUrlValue === undefined) {
                this.throwError(node, `The component in "${this.fileName}" has an empty template URL.`);
            }
            if (!templateUrlValue.endsWith('.html')) {
                this.throwError(node, [
                    `The component in "${this.fileName}" has a template URL with a missing / unsupported file format.`,
                    'Make sure the template URL points to a ".html" file.'
                ].join('\n'));
            }
            this.externalResources.push({
                oldKey: 'templateUrl',
                newKey: 'template',
                node: templateUrlKeyNode,
                urls: [{
                        url: templateUrlValue,
                        node: templateUrlValueNode
                    }]
            });
        }
        if (this.isStyleUrlsDeclaration(node)) {
            const styleUrlsKeyNode = node.name;
            const styleUrlsValueNode = node.initializer;
            const styleUrlsElementNodes = styleUrlsValueNode.elements;
            if (!styleUrlsElementNodes || styleUrlsElementNodes.length === 0) {
                this.throwError(node, `The component in "${this.fileName}" has an empty list of style URLs.`);
            }
            const styleUrls = styleUrlsElementNodes
                .map((element) => {
                return {
                    url: element.text,
                    node: element
                };
            });
            styleUrls.forEach((styleUrl) => {
                if (styleUrl.url === '' || styleUrl.url === 'undefined' || styleUrl.url === undefined) {
                    this.throwError(styleUrl.node, `The component in "${this.fileName}" has an empty style URL.`);
                }
                if (!(styleUrl.url.endsWith('.css') || styleUrl.url.endsWith('.sass') || styleUrl.url.endsWith('.scss'))) {
                    this.throwError(styleUrl.node, [
                        `The component in "${this.fileName}" has a style URL with a missing / unsupported file format.`,
                        'Make sure the style URL points to a ".css", ".scss" or ".sass" file.'
                    ].join('\n'));
                }
            });
            this.externalResources.push({
                oldKey: 'styleUrls',
                newKey: 'styles',
                node: styleUrlsKeyNode,
                urls: styleUrls,
            });
        }
        typescript.forEachChild(node, this.findExternalTemplatesAndStyles.bind(this));
    }
    isComponentDecorator(node) {
        return typescript.isDecorator(node) &&
            node.getChildren().find(typescript.isCallExpression).getChildren().find(typescript.isIdentifier).getText() === 'Component';
    }
    isTemplateUrlDeclaration(node) {
        return typescript.isPropertyAssignment(node) &&
            node.name.getText() === 'templateUrl';
    }
    isStyleUrlsDeclaration(node) {
        return typescript.isPropertyAssignment(node) &&
            node.name.getText() === 'styleUrls';
    }
    replaceNodeContent(fileContent, replacement, node) {
        return `${fileContent.substring(0, node.getStart())}${replacement}${fileContent.substring(node.getEnd())}`;
    }
    throwError(node, message) {
        const { character, line } = this.sourceFile.getLineAndCharacterOfPosition(node.getStart());
        throw new Error([
            message,
            `Details: ${this.filePath} at ${line + 1}:${character + 1}: "${node.getText()}"`
        ].join('\n'));
    }
}
exports.AngularResourceAnalyzer = AngularResourceAnalyzer;
//# sourceMappingURL=angular-resource-analyzer.js.map